'''
Created on 08.09.2018

@author: Daniel Stromer
'''
from CrackDetection.adaptedFrangi import frangi 
from skimage import img_as_ubyte
import cv2
from matplotlib import pyplot as plt
import numpy as np
from newROI import createROI
from fftFunction import fft
import glob
from sklearn.metrics import roc_curve, auc

#mask of fourier
mask_fft = cv2.imread('images/fftmask.tif',0)

image_list=[]
label_list=[]

#Read in images and labels
for filename in glob.glob('images/Label/*.tif'):
    img= cv2.imread(filename,0)
    label_list.append(img)
for filename in glob.glob('images/Raw/*.tif'):
    img= cv2.imread(filename,0)
    image_list.append(img)

#result volumes
size = (len(image_list),image_list[0].shape[0],image_list[0].shape[1])
vol_filtered = np.zeros((size))
vol_frangi_2 = np.zeros((size))
vol_frangi_15 = np.zeros((size))
vol_frangi_25 = np.zeros((size))
vol_frangi_1 = np.zeros((size))
vol_label = np.zeros((size))

#for all images ...
for im in range(len(image_list)):    

    # labeled signal
    _,labeled_crack = cv2.threshold(label_list[im],245,255,cv2.THRESH_BINARY)

    # apply fft for grid finger removal
    img_fft = fft(image_list[im], mask_fft) 
    #equalize histogram
    img_Eq = cv2.equalizeHist(img_fft)
    #create region of interest
    img_roi=createROI(img_Eq)
    #bilateral filter for denoising and edge enhancement
    img_filtered = cv2.bilateralFilter(img_Eq,9,100,100)
    img_filtered = image_list[im]

    #apply eCS  - sigma = 2, n = 1.5
    img_x = img_as_ubyte(frangi(img_filtered,sigma_x = 2, sigma_y = 3, beta1=0.5, beta2=0.02,black_ridges=True))
    img_y = img_as_ubyte(frangi(img_filtered,sigma_x = 3, sigma_y = 2, beta1=0.5, beta2=0.02,black_ridges=True))
    img_fr_15 = np.maximum(img_x,img_y)
    #apply eCS - sigma = 2, n = 2.5
    img_x = img_as_ubyte(frangi(img_filtered,sigma_x = 5, sigma_y = 2, beta1=0.5, beta2=0.02,black_ridges=True))
    img_y = img_as_ubyte(frangi(img_filtered,sigma_x = 2, sigma_y = 5, beta1=0.5, beta2=0.02,black_ridges=True))
    img_fr_25 = np.maximum(img_x,img_y)
    #apply eCS - sigma = 2, n = 2
    img_x = img_as_ubyte(frangi(img_filtered,sigma_x = 2, sigma_y = 4, beta1=0.5, beta2=0.02,black_ridges=True))
    img_y = img_as_ubyte(frangi(img_filtered,sigma_x = 4, sigma_y = 2, beta1=0.5, beta2=0.02,black_ridges=True))
    img_fr_2 = np.maximum(img_x,img_y)
    #apply frangi - sigma = (1,2)
    img_y =  img_as_ubyte(frangi(img_filtered,sigma_x = 1, sigma_y = 1, beta1=0.5, beta2=0.02,black_ridges=True))
    img_x  = img_as_ubyte(frangi(img_filtered,sigma_x = 2, sigma_y = 2, beta1=0.5, beta2=0.02,black_ridges=True))
    img_fr_1 = np.maximum(img_x,img_y)

    #multiply image with roi mask
    img_fr_15_roi = img_fr_15*img_roi 
    img_fr_1_roi = img_fr_1*img_roi 
    img_fr_25_roi = img_fr_25*img_roi 
    img_fr_2_roi = img_fr_2*img_roi 
    img_filtered = img_filtered*img_roi 
    
    #set array
    vol_filtered[im,:,:] = img_filtered
    vol_frangi_2[im,:,:]  = img_fr_2_roi
    vol_frangi_15[im,:,:]  = img_fr_15_roi
    vol_frangi_25[im,:,:]  = img_fr_25_roi
    vol_frangi_1[im,:,:]  = img_fr_1_roi
    vol_label[im,:,:]  = labeled_crack

def evaluation():
    ground_truth_labels = vol_label.ravel()/255.0 # we want to make them into vectors
    
    score_value_filtered = vol_filtered.ravel()/255.0 # we want to make them into vectors
    fpr_filtered, tpr_filtered, thresholds_filtered = roc_curve(ground_truth_labels,score_value_filtered, pos_label=1.0)
    roc_auc_filtered = auc(fpr_filtered,tpr_filtered)
    
    score_value_15 = vol_frangi_15.ravel()/255.0 # we want to make them into vectors
    fpr_15, tpr_15, thresholds_15 = roc_curve(ground_truth_labels,score_value_15, pos_label=1.0)
    roc_auc_15 = auc(fpr_15,tpr_15)
    
    score_value_1 = vol_frangi_1.ravel()/255.0 # we want to make them into vectors
    fpr_1, tpr_1, thresholds_1 = roc_curve(ground_truth_labels,score_value_1, pos_label=1.0)
    roc_auc_1 = auc(fpr_1,tpr_1)
    
    score_value_2 = vol_frangi_2.ravel()/255.0 # we want to make them into vectors
    fpr_2, tpr_2, thresholds_2 = roc_curve(ground_truth_labels,score_value_2, pos_label=1.0)
    roc_auc_2 = auc(fpr_2,tpr_2)
    
    score_value_25 = vol_frangi_25.ravel()/255.0 # we want to make them into vectors
    fpr_25, tpr_25, thresholds_25 = roc_curve(ground_truth_labels,score_value_25, pos_label=1.0)
    roc_auc_25 = auc(fpr_25,tpr_25)

    ax = plt.gca()
    ax.plot(fpr_filtered, tpr_filtered, label='GT (AUC = %0.4f)' % roc_auc_filtered, linewidth=1)
    ax.plot(fpr_1, tpr_1, label='Frangi12 (AUC = %0.4f)' % roc_auc_1, linewidth=1)
    ax.plot(fpr_15, tpr_15, label='eCS1.5 (AUC = %0.4f)' % roc_auc_15, linewidth=1)
    ax.plot(fpr_2, tpr_2, label='eCS2 (AUC = %0.4f)' % roc_auc_2, linewidth=1)
    ax.plot(fpr_25, tpr_25, label='eCS2.5 (AUC = %0.4f)' % roc_auc_25, linewidth=1)
    
    #ax.plot(fpr_3, thresholds_3, label='thresholds', linestyle='dashed')
    ax.plot([0, 1], [0, 1], 'k--')
    ax.set_xlim([0.0, 1.05])
    ax.set_ylim([0.0, 1.05])
    ax.set_xlabel('False Positive Rate')
    ax.set_ylabel('True Positive Rate')
    ax.legend(loc="lower right")
    plt.show()

evaluation()